@@ -140,6 +140,7 @@ nautilus_directory_init (gpointer object, gpointer klass)
 	directory->details->low_priority_queue = nautilus_file_queue_new ();
 	directory->details->extension_queue = nautilus_file_queue_new ();
 	directory->details->idle_queue = nautilus_idle_queue_new ();
+	directory->details->free_space = (guint64)-1;
 }
 
 NautilusDirectory *
@@ -633,6 +634,7 @@ void
 nautilus_directory_add_file (NautilusDirectory *directory, NautilusFile *file)
 {
 	GList *node;
+	gboolean add_to_work_queue;
 
 	g_assert (NAUTILUS_IS_DIRECTORY (directory));
 	g_assert (NAUTILUS_IS_FILE (file));
@@ -647,9 +649,19 @@ nautilus_directory_add_file (NautilusDirectory *directory, NautilusFile *file)
 
 	directory->details->confirmed_file_count++;
 
-	/* Ref if we are monitoring. */
+	add_to_work_queue = FALSE;
 	if (nautilus_directory_is_file_list_monitored (directory)) {
+		/* Ref if we are monitoring, since monitoring owns the file list. */
 		nautilus_file_ref (file);
+		add_to_work_queue = TRUE;
+	} else if (nautilus_directory_has_active_request_for_file (directory, file)) {
+		/* We're waiting for the file in a call_when_ready. Make sure
+		   we add the file to the work queue so that said waiter won't
+		   wait forever for e.g. all files in the directory to be done */
+		add_to_work_queue = TRUE;
+	}
+	
+	if (add_to_work_queue) {
 		nautilus_directory_add_file_to_work_queue (directory, file);
 	}
 }
@@ -1027,24 +1039,6 @@ nautilus_directory_notify_files_added (GList *files)
 	g_hash_table_destroy (parent_directories);
 }
 
-static GList *
-uri_list_to_file_list (GList *uris)
-{
-	GList *l, *file_list;
-	const char *uri;
-	GFile *file;
-		
-	
-	file_list = NULL;
-
-	for (l = uris; l != NULL; l = l->next) {
-		uri = l->data;
-		file = g_file_new_for_uri (uri);
-		file_list = g_list_prepend (file_list, file);
-	}
-	return g_list_reverse (file_list);
-}
-
 static void
 g_file_pair_free (GFilePair *pair)
 {
@@ -1078,7 +1072,7 @@ nautilus_directory_notify_files_added_by_uri (GList *uris)
 {
 	GList *files;
 
-	files = uri_list_to_file_list (uris);
+	files = nautilus_file_list_from_uris (uris);
 	nautilus_directory_notify_files_added (files);
 	eel_g_object_list_free (files);
 }
@@ -1125,7 +1119,7 @@ nautilus_directory_notify_files_changed_by_uri (GList *uris)
 {
 	GList *files;
 
-	files = uri_list_to_file_list (uris);
+	files = nautilus_file_list_from_uris (uris);
 	nautilus_directory_notify_files_changed (files);
 	eel_g_object_list_free (files);
 }
@@ -1183,7 +1177,7 @@ nautilus_directory_notify_files_removed_by_uri (GList *uris)
 {
 	GList *files;
 
-	files = uri_list_to_file_list (uris);
+	files = nautilus_file_list_from_uris (uris);
 	nautilus_directory_notify_files_changed (files);
 	eel_g_object_list_free (files);
 }
@@ -1423,7 +1417,10 @@ nautilus_directory_notify_files_moved (GList *file_pairs)
 			nautilus_file_update_name_and_directory 
 				(file, name, new_directory);
 			g_free (name);
-			
+
+			/* Update file attributes */
+			nautilus_file_invalidate_attributes (file, NAUTILUS_FILE_ATTRIBUTE_INFO);
+
 			hash_table_list_prepend (changed_lists,
 						 old_directory,
 						 file);
@@ -1581,7 +1578,7 @@ nautilus_directory_schedule_metadata_remove_by_uri (GList *uris)
 {
 	GList *files;
 
-	files = uri_list_to_file_list (uris);
+	files = nautilus_file_list_from_uris (uris);
 	nautilus_directory_schedule_metadata_remove (files);
 	eel_g_object_list_free (files);
 }
