@@ -28,20 +28,9 @@
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-vfs-extensions.h>
-#include <libgnomeui/gnome-help.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
-#include <gtk/gtkcelllayout.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcheckbutton.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkcombobox.h>
+#include <gtk/gtk.h>
 #include "nautilus-location-entry.h"
 #include <libnautilus-private/nautilus-global-preferences.h>
 
@@ -201,7 +190,7 @@ connect_to_server (NautilusConnectServerDialog *dialog)
 
 		server = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->server_entry), 0, -1);
 		if (strlen (server) == 0) {
-			eel_show_error_dialog (_("Can't Connect to Server. You must enter a name for the server."), 
+			eel_show_error_dialog (_("Cannot Connect to Server. You must enter a name for the server."), 
 					       _("Please enter a name and try again."), 
 					       GTK_WINDOW (dialog));
 			g_free (server);
@@ -345,10 +334,9 @@ response_callback (NautilusConnectServerDialog *dialog,
 		break;
 	case GTK_RESPONSE_HELP :
 		error = NULL;
-		gnome_help_display_desktop_on_screen (NULL, "user-guide", "user-guide.xml",
-						      "nautilus-server-connect",
-						      gtk_window_get_screen (GTK_WINDOW (dialog)),
-						      &error);
+		gtk_show_uri (gtk_window_get_screen (GTK_WINDOW (dialog)),
+			      "ghelp:user-guide#nautilus-server-connect",
+			      gtk_get_current_event_time (), &error);
 		if (error) {
 			eel_show_error_dialog (_("There was an error displaying help."), error->message,
 					       GTK_WINDOW (dialog));
@@ -899,6 +887,8 @@ nautilus_connect_server_dialog_init (NautilusConnectServerDialog *dialog)
 			    table, TRUE, TRUE, 0);
 
 	dialog->details->uri_entry = nautilus_location_entry_new ();
+	/* hide the clean icon, as it doesn't make sense here */
+	g_object_set (dialog->details->uri_entry, "secondary-icon-name", NULL, NULL);
 	dialog->details->server_entry = gtk_entry_new ();
 	dialog->details->share_entry = gtk_entry_new ();
 	dialog->details->port_entry = gtk_entry_new ();
