@@ -38,11 +38,7 @@
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-pango-extensions.h>
 #include <eel/eel-string.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
@@ -314,7 +310,8 @@ update_icon (NautilusSidebarTitle *sidebar_title)
 						 NAUTILUS_FILE_ATTRIBUTES_FOR_ICON)) {
 		pixbuf = nautilus_file_get_icon_pixbuf (sidebar_title->details->file,
 							sidebar_title->details->best_icon_size,
-							FALSE,
+							TRUE,
+							NAUTILUS_FILE_ICON_FLAGS_USE_THUMBNAILS |
 							NAUTILUS_FILE_ICON_FLAGS_FOR_DRAG_ACCEPT);
 	} else if (sidebar_title->details->determined_icon) {
 		/* We used to know the icon for this file, but now the file says it isn't
@@ -327,14 +324,15 @@ update_icon (NautilusSidebarTitle *sidebar_title)
 	}
 	
 	g_free (icon_name);
-	
-	if (pixbuf != NULL) {
-		sidebar_title->details->determined_icon = TRUE;
-	}
 
 	if (!leave_pixbuf_unchanged) {
 		gtk_image_set_from_pixbuf (GTK_IMAGE (sidebar_title->details->icon), pixbuf);
 	}
+	
+	if (pixbuf != NULL) {
+		sidebar_title->details->determined_icon = TRUE;
+		g_object_unref (pixbuf);
+	}
 }
 
 static void
